@@ -100,8 +100,10 @@ except ImportError:  # pragma: no cover
     ssl = None
 
 
-POTENTIAL_HTTP_PORTS = set([80, 443])
-DEFAULT_HTTP_PORTS = tuple(POTENTIAL_HTTP_PORTS)
+DEFAULT_HTTP_PORTS = frozenset([80])
+POTENTIAL_HTTP_PORTS = set(DEFAULT_HTTP_PORTS)
+DEFAULT_HTTPS_PORTS = frozenset([443])
+POTENTIAL_HTTPS_PORTS = set(DEFAULT_HTTPS_PORTS)
 
 
 class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
@@ -166,6 +168,10 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
         # Now 2 convenient attributes for the HTTPretty API:
 
         # `querystring` holds a dictionary with the parsed query string
+        try:
+            self.path = self.path.encode('iso-8859-1')
+        except UnicodeDecodeError:
+            pass
         self.path = decode_utf8(self.path)
 
         qstring = self.path.split("?", 1)[-1]
@@ -187,7 +193,7 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
         parsed = parse_qs(expanded)
         result = {}
         for k in parsed:
-            result[k] = map(decode_utf8, parsed[k])
+            result[k] = list(map(decode_utf8, parsed[k]))
 
         return result
 
@@ -205,6 +211,7 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
 
         do_parse = PARSING_FUNCTIONS.get(content_type, FALLBACK_FUNCTION)
         try:
+            body = decode_utf8(body)
             return do_parse(body)
         except:
             return body
@@ -283,7 +290,7 @@ class fakesock(object):
         def connect(self, address):
             self._address = (self._host, self._port) = address
             self._closed = False
-            self.is_http = self._port in POTENTIAL_HTTP_PORTS
+            self.is_http = self._port in POTENTIAL_HTTP_PORTS | POTENTIAL_HTTPS_PORTS
 
             if not self.is_http:
                 self.truesock.connect(self._address)
@@ -344,7 +351,7 @@ class fakesock(object):
 
             try:
                 requestline, _ = data.split(b'\r\n', 1)
-                method, path, version = parse_requestline(requestline)
+                method, path, version = parse_requestline(decode_utf8(requestline))
                 is_parsing_headers = True
             except ValueError:
                 is_parsing_headers = False
@@ -358,10 +365,10 @@ class fakesock(object):
             if not is_parsing_headers:
                 if len(self._sent_data) > 1:
                     headers = utf8(last_requestline(self._sent_data))
-                    meta = dict(self._entry.request.headers)
+                    meta = self._entry.request.headers
                     body = utf8(self._sent_data[-1])
                     if meta.get('transfer-encoding', '') == 'chunked':
-                        if not body.isdigit() and body != '\r\n' and body != '0\r\n\r\n':
+                        if not body.isdigit() and body != b'\r\n' and body != b'0\r\n\r\n':
                             self._entry.request.body += body
                     else:
                         self._entry.request.body += body
@@ -372,7 +379,7 @@ class fakesock(object):
             # path might come with
             s = urlsplit(path)
             POTENTIAL_HTTP_PORTS.add(int(s.port or 80))
-            headers, body = map(utf8, data.split(b'\r\n\r\n', 1))
+            headers, body = list(map(utf8, data.split(b'\r\n\r\n', 1)))
 
             request = httpretty.historify_request(headers, body)
 
@@ -393,7 +400,7 @@ class fakesock(object):
         def debug(self, func, *a, **kw):
             if self.is_http:
                 frame = inspect.stack()[0][0]
-                lines = map(utf8, traceback.format_stack(frame))
+                lines = list(map(utf8, traceback.format_stack(frame)))
 
                 message = [
                     "HTTPretty intercepted and unexpected socket method call.",
@@ -630,7 +637,12 @@ class URIInfo(BaseClass):
         self.port = port or 80
         self.path = path or ''
         self.query = query or ''
-        self.scheme = scheme or (self.port == 443 and "https" or "http")
+        if scheme:
+            self.scheme = scheme
+        elif self.port in POTENTIAL_HTTPS_PORTS:
+            self.scheme = 'https'
+        else:
+            self.scheme = 'http'
         self.fragment = fragment or ''
         self.last_request = last_request
 
@@ -682,7 +694,8 @@ class URIInfo(BaseClass):
 
     def get_full_domain(self):
         hostname = decode_utf8(self.hostname)
-        if self.port not in DEFAULT_HTTP_PORTS:
+        # Port 80/443 should not be appended to the url
+        if self.port not in DEFAULT_HTTP_PORTS | DEFAULT_HTTPS_PORTS:
             return ":".join([hostname, str(self.port)])
 
         return hostname
@@ -690,7 +703,10 @@ class URIInfo(BaseClass):
     @classmethod
     def from_uri(cls, uri, entry):
         result = urlsplit(uri)
-        POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
+        if result.scheme == 'https':
+            POTENTIAL_HTTPS_PORTS.add(int(result.port or 443))
+        else:
+            POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
         return cls(result.username,
                    result.password,
                    result.hostname,
@@ -710,6 +726,11 @@ class URIMatcher(object):
         self._match_querystring = match_querystring
         if type(uri).__name__ == 'SRE_Pattern':
             self.regex = uri
+            result = urlsplit(uri.pattern)
+            if result.scheme == 'https':
+                POTENTIAL_HTTPS_PORTS.add(int(result.port or 443))
+            else:
+                POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
         else:
             self.info = URIInfo.from_uri(uri, entries)
 
@@ -807,12 +828,12 @@ class httpretty(HttpBaseClass):
                     'uri': uri,
                     'method': request.method,
                     'headers': dict(request.headers),
-                    'body': request.body,
+                    'body': decode_utf8(request.body),
                     'querystring': request.querystring
                 },
                 'response': {
                     'status': response.status,
-                    'body': response.data,
+                    'body': decode_utf8(response.data),
                     'headers': dict(response.headers)
                 }
             })
@@ -843,6 +864,8 @@ class httpretty(HttpBaseClass):
 
     @classmethod
     def reset(cls):
+        POTENTIAL_HTTP_PORTS.intersection_update(DEFAULT_HTTP_PORTS)
+        POTENTIAL_HTTPS_PORTS.intersection_update(DEFAULT_HTTPS_PORTS)
         cls._entries.clear()
         cls.latest_requests = []
         cls.last_request = HTTPrettyRequestEmpty()
